package glk.poorglk;
import glk.*;
import java.io.File;
import java.io.IOException;
class PoorFileRef extends PoorObject implements frefid_t
{
    protected int usage;
    protected File myfile;
    
    public static PoorFileRef create_temp(PoorGlk glk, int usage, int rock)
    {
	return new PoorFileRef(glk, usage, rock);
    }

    public static PoorFileRef create_by_name(PoorGlk glk, int usage, String name, int rock)
    {
	return new PoorFileRef(glk, usage, rock, name);
    }
    
    public static PoorFileRef create_by_prompt(PoorGlk glk, int usage, int fmode, int rock)
    {
	String prompt = "Oops!";
	switch (usage & Glk.fileusage_TypeMask) {
	case Glk.fileusage_SavedGame:
	    prompt = "Enter name for save file: ";
	    break;

	case Glk.fileusage_Transcript:
	    prompt = "Enter file name for transcript: ";
	    break;

	case Glk.fileusage_InputRecord:
	    prompt = "Enter name for keystroke file: ";
	    break;

	case Glk.fileusage_Data:
	    prompt = "Enter name for file: ";
	    break;
	}
	System.out.println(prompt);
	try {
	    String name = glk.getKeyboardReader().readLine();
	    return new PoorFileRef(glk, usage, rock, name);
	}
	catch (IOException ioe) {
	    /*can't read KB, must be in bad shape!*/
	    ioe.printStackTrace();
	    return null;
	}
    }
    
    public static PoorFileRef create_from_fileref(PoorGlk glk, int usage, frefid_t fref, int rock) 
    {
	return new PoorFileRef(glk, usage, (PoorFileRef)fref, rock);
    }

    /* package */ File getFile()
    {
	return myfile;
    }

    PoorFileRef(PoorGlk glk, int usage, int rock) 
    {
	super(glk, rock);
	this.usage = usage;
	try {
	    myfile = File.createTempFile("poorglk", null);
	    myfile.delete();
	    myfile.deleteOnExit();
	}
	catch (IOException ioe) {
	    ioe.printStackTrace();
	    /* Sigh */
	}
    }

    PoorFileRef(PoorGlk glk, int usage, int rock, String name) 
    {
	super(glk, rock);
	this.usage = usage;
	myfile = new File(name);
    }

    PoorFileRef(PoorGlk glk, int usage, PoorFileRef copyref, int rock) 
    {
	super(glk, rock);
	this.usage = usage;
	myfile = copyref.myfile;
    }

    public void destroy()
    {
	myfile = null;
    }

    public void delete_file()
    {
	myfile.delete();
    }

    public int does_file_exist()
    {
	return myfile.exists()?Glk.TRUE:Glk.FALSE;
    }

    /* package */boolean isBinary() {
	return (usage & Glk.fileusage_TextMode) == Glk.fileusage_BinaryMode;
    }
}
