package glk.poorglk;
import glk.*;
import java.io.UnsupportedEncodingException;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.RandomAccessFile;
class PoorFileStream extends PoorStream
{
    protected byte[] buffer;
    PoorFileRef pfref;
    RandomAccessFile raf;

    PoorFileStream(PoorGlk glk, frefid_t fref, int fmode, int rock) 
	throws IOException {
	super(glk, fmode, rock);
	if ((fmode != Glk.filemode_Write) && (fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite) && (fmode != Glk.filemode_WriteAppend)) {
	    throw new IllegalArgumentException("File stream created with bogus filemode " + fmode);
	}
	pfref = (PoorFileRef)fref;
	if ((fmode == Glk.filemode_Read) &&
	    ((pfref.does_file_exist() == Glk.FALSE))) {
	    throw new FileNotFoundException();
	}
	pfref.getFile().createNewFile(); /* does nothing if file exists */

	String mode;
	if (fmode == Glk.filemode_Read)
	    mode = "r";
	else 
	    mode = "rw";

	raf = new RandomAccessFile(pfref.getFile(), mode);
	if (fmode == Glk.filemode_WriteAppend)
	    raf.seek(raf.length());
    }

    public void put_char(char ch)
    {
	try {
	    if (fmode == Glk.filemode_Read)
		System.err.println("put_char called on file stream not open for writing");
	    else if (pfref.isBinary()) {
		raf.writeByte((int)ch);
	    }
	    else {
		if ((ch == '\n' || ch == '\r'))
		    raf.writeBytes(new String(System.getProperty("line.separator", "\n").getBytes()));
		else
		    raf.write((int)ch);
	    }
	    writechars++;
	}
	catch (IOException ioe) {
	    ioe.printStackTrace();
	}
    }

    public void put_string(String s)
    {
	if (fmode == Glk.filemode_Read)
	    System.err.println("put_string called on file stream not open for writing");
	else if (pfref.isBinary()) {
	    try {
		raf.writeBytes(s); /* strips high byte-- fine for GLK */
		writechars += s.length();
	    }
	    catch (IOException ioe) {
		ioe.printStackTrace();
	    }
	}
	else {
	    /* SLOW-- FIX ME */
	    for (int i = 0; i < s.length(); i++)
		put_char(s.charAt(i));
	}
    }

    public void put_buffer(char []ca, int length)
    {
	if (fmode == Glk.filemode_Read)
	    System.err.println("put_buffer called on file stream not open for writing");
	else if (pfref.isBinary()) {
	    try {
		raf.writeBytes(new String(ca, 0, length)); /* strips high byte-- fine for GLK */
		writechars += length;
	    }
	    catch (IOException ioe) {
		ioe.printStackTrace();
	    }
	}
	else {
	    /* SLOW-- FIX ME */
	    for (int i = 0; i < length; i++)
		put_char(ca[i]);
	}
    }

    public void put_buffer(byte []ba, int length)
    {
	if (fmode == Glk.filemode_Read)
	    System.err.println("put_buffer called on file stream not open for writing");
	else if (pfref.isBinary()) {
	    try {
		raf.write(ba, 0, length);
		writechars += length;
	    }
	    catch (IOException ioe) {
		ioe.printStackTrace();
	    }
	}
	else {
	    /* SLOW-- FIX ME */
	    for (int i = 0; i < length; i++)
		put_char((char)ba[i]);
	}
    }
    
    public int get_char()
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_char called on file stream not open for reading");
	    return -1;
	}

	try {
	    int ch = raf.read();
	    if (ch != -1)
		readchars++;
	    if (!pfref.isBinary() && (ch == (int)'\r')) {
		long savepos = raf.getFilePointer();
		ch = raf.read();
		if (ch != (int)'\n')
		    raf.seek(savepos);
		ch = (int)'\n';
	    }
	    return ch;
	}
	catch (IOException ioe) {
	    ioe.printStackTrace();
	    return -1;
	}
    }

    public int get_line(char []buf, int len)
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_line called on file stream not open for reading");
	    return -1;
	}
	else {
	    /* fixme, slow */
	    int actcount = 0;
	    for (int i = 0; i < len; i++) {
		int ch;
		ch = get_char();
		if (ch == -1)
		    break;
		actcount++;
		buf[i] = (char)ch;
		if (buf[i] == '\n')
		    break;
	    }
	    return actcount;
	}
    }

    public int get_line(byte []buf, int len)
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_line called on file stream not open for reading");
	    return -1;
	}
	else {
	    /* fixme, slow */
	    int actcount = 0;
	    for (int i = 0; i < len; i++) {
		int ch;
		ch = get_char();
		if (ch == -1)
		    break;
		actcount++;
		buf[i] = (byte)ch;
		if ((char)ch == '\n')
		    break;
	    }
	    return actcount;
	}
    }

    public int get_buffer(char []buf, int len)
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_buffer called on file stream not open for reading");
	    return -1;
	}
	else if (!pfref.isBinary()) {
	    try {
		byte ba[] = new byte[len];
		int nchars = raf.read(ba);
		for (int i = 0; i < nchars; i++)
		    buf[i] = (char)(ba[i] & 0xFF);
		readchars += nchars;
		return nchars;
	    }
	    catch (IOException ioe) {
		ioe.printStackTrace();
		return -1;
	    }

	}
	else {
	    /* fixme, slow */
	    int actcount = 0;
	    for (int i = 0; i < len; i++) {
		int ch;
		ch = get_char();
		if (ch == -1)
		    break;
		actcount++;
		buf[i] = (char)ch;
	    }
	    return actcount;
	}
    }

    public int get_buffer(byte []buf, int len)
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_buffer called on file stream not open for reading");
	    return -1;
	}
	else if (!pfref.isBinary()) {
	    try {
		int nchars = raf.read(buf, 0, len);
		readchars += nchars;
		return nchars;
	    }
	    catch (IOException ioe) {
		ioe.printStackTrace();
		return -1;
	    }
	}
	else {
	    /* fixme, slow */
	    int actcount = 0;
	    for (int i = 0; i < len; i++) {
		int ch;
		ch = get_char();
		if (ch == -1)
		    break;
		actcount++;
		buf[i] = (byte)ch;
	    }
	    return actcount;
	}
    }

    public void set_position(int pos, int seekmode)
    {
	try {
	    long posfromstart = pos&0xFFFFFFFF;
	    
	    if (seekmode == Glk.seekmode_Current)
		posfromstart += raf.getFilePointer();
	    else if (seekmode == Glk.seekmode_End) 
		posfromstart = raf.length() - posfromstart;
	    
	    if ((posfromstart > raf.length()) || (posfromstart < 0))
		System.err.println("set_position called with position outside buffer");
	    else
		raf.seek(posfromstart);
	}
	catch (IOException ioe) {
	    ioe.printStackTrace();
	}
    }

    public int get_position()
    {
	try {
	    return (int)raf.getFilePointer();
	}
	catch (IOException ioe) {
	    ioe.printStackTrace();
	    return 0;
	}
    }

    public void cleanup(stream_result_t result)
    {
	super.cleanup(result);
	try {
	    raf.close();
	    raf = null;
	    pfref = null;
	}
	catch (IOException ioe) {
	    ioe.printStackTrace();
	}
    }
}
