package glk.poorglk;
import glk.*;
import java.io.UnsupportedEncodingException;
class PoorMemoryStream extends PoorStream
{
    
    protected byte[] buffer;
    Object bufferrock;
    protected long buflen;
    protected long position;

    PoorMemoryStream(PoorGlk glk, byte []buf, int buflen, int fmode, int rock) {
	super(glk, fmode, rock);
	if ((fmode != Glk.filemode_Write) && (fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite))
	    throw new IllegalArgumentException("Memory stream created with bogus filemode " + fmode);
	this.buffer = buf;
	this.buflen = buflen&0xFFFFFFFF;
	this.position = 0;
	this.bufferrock = glk.arrayreg.my_vm_reg_array(buf, buflen,
						   glk.memstream_typecode);
    }

    public void put_char(char ch)
    {
	if ((fmode != Glk.filemode_Write) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("put_char called on memory stream not open for writing");
	    
	}
	else {
	    put_string(new Character(ch).toString());
	}
    }

    public void put_string(String s)
    {
	if ((fmode != Glk.filemode_Write) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("put_string called on memory stream not open for writing");
	    
	}
	else {
	    try {
		byte[]ba = s.getBytes("ISO-8859-1");
		put_buffer(ba, ba.length);
	    }
	    catch (UnsupportedEncodingException ioe) {}
	}
    }

    public void put_buffer(char []ca, int length)
    {
	if ((fmode != Glk.filemode_Write) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("put_buffer called on memory stream not open for writing");
	    
	}
	else {
	    try {
		byte[]ba = new String(ca, 0, length).getBytes("ISO-8859-1");
		put_buffer(ba, ba.length);
	    }
	    catch (UnsupportedEncodingException ioe) {}
	}
    }

    public void put_buffer(byte []ba, int length)
    {
	if ((fmode != Glk.filemode_Write) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("put_buffer called on memory stream not open for writing");
	    
	}
	else {
	    if (buffer != null) {
		for (int i = 0; (i < length) && (position < buflen);
		     position++, i++) {
		    buffer[(int)position] = ba[i];
		}
	    }
	    writechars += length;
	}
    }
    
    public int get_char()
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_char called on memory stream not open for reading");
	    return -1;
	}
	else if ((buffer == null) || (position >= buflen))
	    return -1;
	else {
	    readchars++;
	    return (int)(buffer[(int)position++]&0xFF);
	}
    }

    public int get_line(char []buf, int len)
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_line called on memory stream not open for reading");
	    return -1;
	}
	else {
	    int actcount = 0;
	    for (int i = 0; i < len; i++) {
		int ch;
		ch = get_char();
		if (ch == -1)
		    break;
		actcount++;
		buf[i] = (char)ch;
		if (buf[i] == '\n')
		    break;
	    }
	    return actcount;
	}
    }

    public int get_line(byte []buf, int len)
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_line called on memory stream not open for reading");
	    return -1;
	}
	else {
	    int actcount = 0;
	    for (int i = 0; i < len; i++) {
		int ch;
		ch = get_char();
		if (ch == -1)
		    break;
		actcount++;
		buf[i] = (byte)ch;
		if ((char)ch == '\n')
		    break;
	    }
	    return actcount;
	}
    }

    public int get_buffer(char []buf, int len)
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_buffer called on memory stream not open for reading");
	    return -1;
	}
	else {
	    int actcount = 0;
	    for (int i = 0; i < len; i++) {
		int ch;
		ch = get_char();
		if (ch == -1)
		    break;
		actcount++;
		buf[i] = (char)ch;
	    }
	    return actcount;
	}
    }

    public int get_buffer(byte []buf, int len)
    {
	if ((fmode != Glk.filemode_Read) &&
	    (fmode != Glk.filemode_ReadWrite)) {
	    System.err.println("get_buffer called on memory stream not open for reading");
	    return -1;
	}
	else {
	    int actcount = 0;
	    for (int i = 0; i < len; i++) {
		int ch;
		ch = get_char();
		if (ch == -1)
		    break;
		actcount++;
		buf[i] = (byte)ch;
	    }
	    return actcount;
	}
    }

    public void set_position(int pos, int seekmode)
    {
	long posfromstart = pos&0xFFFFFFFF;
	
	if (seekmode == Glk.seekmode_Current)
	    posfromstart += position;
	else if (seekmode == Glk.seekmode_End) 
	    posfromstart = buflen - posfromstart;
	
	if ((posfromstart > buflen) || (posfromstart < 0))
	    System.err.println("set_position called with position outside buffer");
	else
	    position = posfromstart;
    }

    public int get_position()
    {
	return (int)position;
    }
    
    public void cleanup(stream_result_t result)
    {
	super.cleanup(result);
	glk.arrayreg.my_vm_unreg_array(buffer, (int)buflen,
				       glk.memstream_typecode,
				       bufferrock);
    }
}
